
#include "xbasic_types.h"
#include "xio.h"

#define W 640
#define VGA_START 0x00800000
#define BRAM_START 0x01800000

// Transfer a section of "line" starting at pixel //
// "start" and ending at  pixel  "end"  from  the //
// block RAMs to the SRAM.                        //
void write_video(int start, int end, int line)
{
  int nPixs;
  Xuint32 luma_4pixels;
  Xuint32 bram_addr;
  Xuint32 vga_addr;

  nPixs = (end - start);
  vga_addr = VGA_START + (start>>1) + W*line; 
  bram_addr = BRAM_START + (start>>1);

  while (nPixs > 0)
  {
    luma_4pixels = XIo_In32(bram_addr+0);
    XIo_Out32(vga_addr+0, luma_4pixels);

    luma_4pixels = XIo_In32(bram_addr+4);
    XIo_Out32(vga_addr+4, luma_4pixels);

    luma_4pixels = XIo_In32(bram_addr+8);
    XIo_Out32(vga_addr+8, luma_4pixels);

    luma_4pixels = XIo_In32(bram_addr+12);
    XIo_Out32(vga_addr+12, luma_4pixels);

    bram_addr += 16;
    vga_addr += 16;

    // Skip pixels //
    nPixs -=32;

    // If we lose vertical synchronism in the meantime //
    // then break from the "while" and return          //
    if (!XIo_In32(0x01802FFC))
      break;
  }
}
