
-- Component: MULTIPLEXOR --------------------------------------------

library IEEE;
use IEEE.std_logic_1164.all;
use IEEE.std_logic_arith.all;
use IEEE.std_logic_unsigned.all;

entity mux is 
    port( rst, sLine: in std_logic;
	  load, result: in std_logic_vector( 15 downto 0 );
	  output: out std_logic_vector( 15 downto 0 )
    );
end mux;

architecture mux_arch of mux is
begin
    process( rst, sLine, load, result )
    begin
	if( rst = '1' ) then 
	    output <= (others => '0');	    
	elsif sLine = '0' then 
	    output <= load;	       
	else 
	    output <= result;
	end if;
    end process;
end mux_arch;

-- Component: 4to1 MULTIPLEXOR -----------------------------------

library IEEE;
use IEEE.std_logic_1164.all;
use IEEE.std_logic_arith.all;
use IEEE.std_logic_unsigned.all;

entity mux4to1 is 
    port( rst: in std_logic;
          sel: in std_logic_vector(1 downto 0);
	  load00, load01, load10, load11: in std_logic_vector( 9 downto 0 );
	  output: out std_logic_vector( 9 downto 0 )
    );
end mux4to1;

architecture mux4_arch of mux4to1 is
begin
    process( rst, sel, load00, load01, load10, load11)
    begin
	if( rst = '1' ) then 
          output <= (others => '0');	    
	elsif sel = "00" then 
          output <= load00;	       
	elsif sel = "01" then
          output <= load01;
        elsif sel = "10" then
          output <= load10;
        elsif sel="11" then
          output <= load11;
	end if;
    end process;
end mux4_arch;
	
-- Component: ADDER ----------------------------------------------

library IEEE;
use IEEE.std_logic_1164.all;
use IEEE.std_logic_arith.all;
use IEEE.std_logic_unsigned.all;

entity adder is
    port( rst: in std_logic;
	  x, y: in std_logic_vector( 15 downto 0 );
	  aout: out std_logic_vector( 15 downto 0 )
    );
end adder;

architecture adder_arch of adder is
begin
    process( rst, x, y )
    begin
	if( rst = '1' ) then  
	    aout <= (others => '0');	
        else 	   
            aout <= x + y;
	end if;
    end process;
end adder_arch;

-- Component: REGISTER ---------------------------------------------------

library IEEE;
use IEEE.std_logic_1164.all;
use IEEE.std_logic_arith.all;
use IEEE.std_logic_unsigned.all;

entity regis is
    port( rst, clk, load: in std_logic;
	  input: in std_logic_vector( 15 downto 0 );
	  output: out std_logic_vector( 15 downto 0 )
    );
end regis;

architecture regis_arc of regis is
begin
    process( rst, clk, load, input )
    begin
	if( rst = '1' ) then 
	    output <= (others => '0');
	elsif( clk'event and clk = '1') then
	    if( load = '1' ) then	
	        output <= input;
	    end if;
        end if;
    end process;
end regis_arc;

-- Component: 19-bit REGISTER ------------------------------------------------------------

library IEEE;
use IEEE.std_logic_1164.all;
use IEEE.std_logic_arith.all;
use IEEE.std_logic_unsigned.all;

entity regis20 is
    port( rst, clk, load: in std_logic;
	  input: in std_logic_vector( 19 downto 0 );
	  output: out std_logic_vector( 19 downto 0 )
    );
end regis20;

architecture regis20_arc of regis20 is
begin
    process( rst, clk, load, input )
    begin
	if( rst = '1' ) then 
	    output <= (others => '0');
	elsif( clk'event and clk = '1') then
	    if( load = '1' ) then	
	        output <= input;
	    end if;
        end if;
    end process;
end regis20_arc;


-- Component: COUNTER ---------------------------------------------------

library IEEE;
use IEEE.std_logic_1164.all;
use IEEE.std_logic_arith.all;
use IEEE.std_logic_unsigned.all;

entity counter is
    port( rst, clk: in std_logic;
	  output: out std_logic_vector( 10 downto 0 )
    );
end counter;

architecture counter_arc of counter is
  signal count : std_logic_vector(10 downto 0);

begin
    process( rst, clk )
    begin
	if( rst = '1' ) then 
	    count <= (others => '0');
	elsif( clk'event and clk = '1') then
            count <= count + 1;
        end if;
    end process;

    output <= count;
    
end counter_arc;

-- component: Multiplier ------------------------------------------------

LIBRARY ieee;
USE ieee.std_logic_1164.ALL;
USE ieee.numeric_std.all;
USE ieee.std_logic_signed.all;


ENTITY mymult IS
	port (
        rst, clk: in std_logic;
	a: IN std_logic_VECTOR(15 downto 0);
	b: IN std_logic_VECTOR(15 downto 0);
	o: OUT std_logic_VECTOR(31 downto 0));
END mymult;

ARCHITECTURE mymult_a OF mymult IS

  signal aint: signed (15 downto 0);
  signal bint: signed (15 downto 0);

begin

  aint <= signed(a);
  bint <= signed(b);

  o <= std_logic_vector(aint*bint);

END mymult_a;


-- Top level design. Puts all components together

library IEEE;
use IEEE.std_logic_1164.all;
use IEEE.std_logic_arith.all;
use IEEE.std_logic_unsigned.all;
-- use work.all;

entity warper is 
    port(   rst, clk: in std_logic; 
	    linecount: in std_logic_vector( 9 downto 0 );
            pixelcount: in std_logic_vector( 10 downto 0 );
--            bramDataIn: in std_logic_vector( 15 downto 0 );
--            bramAddr: out std_logic_vector( 9 downto 0 );
--            x_out: out std_logic_vector(15 downto 0 );
	    addr: out std_logic_vector( 19 downto 0 ) 
    ); 
end warper; 

architecture warper_arch of warper is

component mux is 
    port(	rst, sLine: in std_logic;
		load, result: in std_logic_vector( 15 downto 0 );
		output: out std_logic_vector( 15 downto 0 )
    );
end component;

component mux4to1 is 
    port( rst: in std_logic;
          sel: in std_logic_vector(1 downto 0);
	  load00, load01, load10, load11: in std_logic_vector( 9 downto 0 );
	  output: out std_logic_vector( 9 downto 0 )
    );
end component;

component adder is
    port( 	rst: in std_logic; 
		x, y: in std_logic_vector( 15 downto 0 );
		aout: out std_logic_vector( 15 downto 0 )
    );
end component;

component inverter is
    port( 	rst: in std_logic; 
		x: in std_logic_vector( 15 downto 0 );
		o: out std_logic_vector( 15 downto 0 )
    );
end component;

component regis is
    port(	rst, clk, load: in std_logic;
		input: in std_logic_vector( 15 downto 0 );
		output: out std_logic_vector( 15 downto 0 )
    );
end component;

component regis20 is
    port( rst, clk, load: in std_logic;
	  input: in std_logic_vector( 19 downto 0 );
	  output: out std_logic_vector( 19 downto 0 )
    );
end component;

component mymult is
    port(	rst, clk: in std_logic;
                a, b: in std_logic_vector( 15 downto 0 );
                o: out std_logic_vector(31 downto 0 )
    );
end component;


-- These are the key components to the math of the warper

constant yOriginTranslatePlusOne: std_logic_vector(15 downto 0):=X"F001";
            --:= "0011110000"&"000001";  -- 240 . .1 (fixed point)
constant xOriginTranslateAndLineMax: std_logic_vector(15 downto 0):=X"0000";--:=X"D800";
            --:= "0001010000"&"000000";  -- 320 - 240 = 80 (fixed point)
constant bramAddrXOffset: std_logic_vector(9 downto 0)
            := "0000000000";            -- 0
constant bramAddrYOffset: std_logic_vector(9 downto 0)
            := "0100000000";            -- 256
constant bramAddrSinOffset: std_logic_vector(9 downto 0)
            := "1000000000";            -- 512 (address doesn't care about 2's comp)
constant unused: std_logic_vector(9 downto 0 )
            := "0000000000";            -- Just for the 4to1 mux
constant pixelCountSetup: std_logic_vector(10 downto 0):= "01100010000";
--            := "00000000000";           -- 0
constant pixelCountStart: std_logic_vector(10 downto 0):= "00000000000";
--            := "00010001100";           -- ~140
constant pixelCountStop : std_logic_vector(10 downto 0):= "01010000000";
--            := "01100001101";           -- 781

type ram_type is array(0 to 239) of std_logic_vector(15 downto 0);
constant X_RAM : ram_type :=
  (
    X"5367",
X"5337",X"5307",X"52D8",X"52A8",X"5278",
X"5248",X"5219",X"51E9",X"51BA",X"518B",
X"515B",X"512C",X"50FD",X"50CE",X"509F",
X"5070",X"5041",X"5012",X"4FE4",X"4FB5",
X"4F87",X"4F58",X"4F2A",X"4EFC",X"4ECD",
X"4E9F",X"4E71",X"4E43",X"4E15",X"4DE7",
X"4DBA",X"4D8C",X"4D5F",X"4D31",X"4D04",
X"4CD6",X"4CA9",X"4C7C",X"4C4F",X"4C22",
X"4BF5",X"4BC8",X"4B9B",X"4B6F",X"4B42",
X"4B16",X"4AE9",X"4ABD",X"4A91",X"4A65",
X"4A39",X"4A0D",X"49E1",X"49B5",X"498A",
X"495E",X"4933",X"4907",X"48DC",X"48B1",
X"4886",X"485B",X"4830",X"4805",X"47DA",
X"47B0",X"4785",X"475B",X"4730",X"4706",
X"46DC",X"46B2",X"4688",X"465E",X"4634",
X"460B",X"45E1",X"45B8",X"458E",X"4565",
X"453C",X"4513",X"44EA",X"44C1",X"4498",
X"4470",X"4447",X"441F",X"43F7",X"43CE",
X"43A6",X"437E",X"4357",X"432F",X"4307",
X"42E0",X"42B8",X"4291",X"426A",X"4243",
X"421C",X"41F5",X"41CE",X"41A7",X"4181",
X"415A",X"4134",X"410E",X"40E8",X"40C2",
X"409C",X"4076",X"4051",X"402B",X"4006",
X"3FE1",X"3FBB",X"3F96",X"3F72",X"3F4D",
X"3F28",X"3F04",X"3EDF",X"3EBB",X"3E97",
X"3E73",X"3E4F",X"3E2B",X"3E07",X"3DE4",
X"3DC0",X"3D9D",X"3D7A",X"3D57",X"3D34",
X"3D11",X"3CEE",X"3CCC",X"3CAA",X"3C87",
X"3C65",X"3C43",X"3C21",X"3C00",X"3BDE",
X"3BBC",X"3B9B",X"3B7A",X"3B59",X"3B38",
X"3B17",X"3AF6",X"3AD6",X"3AB5",X"3A95",
X"3A75",X"3A55",X"3A35",X"3A15",X"39F6",
X"39D6",X"39B7",X"3998",X"3979",X"395A",
X"393B",X"391C",X"38FE",X"38DF",X"38C1",
X"38A3",X"3885",X"3867",X"384A",X"382C",
X"380F",X"37F2",X"37D5",X"37B8",X"379B",
X"377E",X"3762",X"3745",X"3729",X"370D",
X"36F1",X"36D5",X"36BA",X"369E",X"3683",
X"3668",X"364D",X"3632",X"3617",X"35FD",
X"35E2",X"35C8",X"35AE",X"3594",X"357A",
X"3560",X"3547",X"352D",X"3514",X"34FB",
X"34E2",X"34C9",X"34B0",X"3498",X"3480",
X"3467",X"344F",X"3437",X"3420",X"3408",
X"33F1",X"33D9",X"33C2",X"33AB",X"3394",
X"337E",X"3367",X"3351",X"333B",X"3324",
X"330F",X"32F9",X"32E3",X"32CE",X"32B8",
X"32A3",X"328E",X"3279",X"3265",X"3250",
X"323C",X"3228",X"3213",X"3200"
    );

constant Y_RAM : ram_type :=
  (
X"0F9A",
X"0FA5",X"0FAF",X"0FBA",X"0FC5",X"0FD0",
X"0FDB",X"0FE6",X"0FF1",X"0FFC",X"1007",
X"1012",X"101D",X"1028",X"1034",X"103F",
X"104B",X"1056",X"1061",X"106D",X"1079",
X"1084",X"1090",X"109C",X"10A8",X"10B3",
X"10BF",X"10CB",X"10D7",X"10E3",X"10EF",
X"10FC",X"1108",X"1114",X"1121",X"112D",
X"1139",X"1146",X"1152",X"115F",X"116C",
X"1179",X"1185",X"1192",X"119F",X"11AC",
X"11B9",X"11C6",X"11D3",X"11E0",X"11EE",
X"11FB",X"1208",X"1216",X"1223",X"1231",
X"123F",X"124C",X"125A",X"1268",X"1276",
X"1284",X"1292",X"12A0",X"12AE",X"12BC",
X"12CA",X"12D9",X"12E7",X"12F5",X"1304",
X"1312",X"1321",X"1330",X"133F",X"134D",
X"135C",X"136B",X"137A",X"1389",X"1399",
X"13A8",X"13B7",X"13C7",X"13D6",X"13E5",
X"13F5",X"1405",X"1414",X"1424",X"1434",
X"1444",X"1454",X"1464",X"1474",X"1485",
X"1495",X"14A5",X"14B6",X"14C6",X"14D7",
X"14E8",X"14F8",X"1509",X"151A",X"152B",
X"153C",X"154D",X"155E",X"1570",X"1581",
X"1592",X"15A4",X"15B6",X"15C7",X"15D9",
X"15EB",X"15FD",X"160F",X"1621",X"1633",
X"1645",X"1658",X"166A",X"167C",X"168F",
X"16A2",X"16B4",X"16C7",X"16DA",X"16ED",
X"1700",X"1713",X"1726",X"173A",X"174D",
X"1761",X"1774",X"1788",X"179C",X"17AF",
X"17C3",X"17D7",X"17EB",X"1800",X"1814",
X"1828",X"183D",X"1851",X"1866",X"187A",
X"188F",X"18A4",X"18B9",X"18CE",X"18E3",
X"18F8",X"190E",X"1923",X"1939",X"194E",
X"1964",X"197A",X"1990",X"19A6",X"19BC",
X"19D2",X"19E8",X"19FE",X"1A15",X"1A2B",
X"1A42",X"1A59",X"1A6F",X"1A86",X"1A9D",
X"1AB4",X"1ACC",X"1AE3",X"1AFA",X"1B12",
X"1B29",X"1B41",X"1B59",X"1B71",X"1B89",
X"1BA1",X"1BB9",X"1BD1",X"1BE9",X"1C02",
X"1C1A",X"1C33",X"1C4C",X"1C65",X"1C7E",
X"1C97",X"1CB0",X"1CC9",X"1CE2",X"1CFC",
X"1D15",X"1D2F",X"1D49",X"1D62",X"1D7C",
X"1D96",X"1DB1",X"1DCB",X"1DE5",X"1E00",
X"1E1A",X"1E35",X"1E4F",X"1E6A",X"1E85",
X"1EA0",X"1EBB",X"1ED7",X"1EF2",X"1F0D",
X"1F29",X"1F45",X"1F60",X"1F7C",X"1F98",
X"1FB4",X"1FD0",X"1FED",X"2009",X"2025",
X"2042",X"205F",X"207B",X"2098",X"20B5",
X"20D2",X"20F0",X"210D",X"212A"
    );

constant SIN_RAM : ram_type :=
  (
X"004B",
X"004C",X"004C",X"004C",X"004D",X"004D",
X"004D",X"004E",X"004E",X"004F",X"004F",
X"004F",X"0050",X"0050",X"0051",X"0051",
X"0051",X"0052",X"0052",X"0053",X"0053",
X"0053",X"0054",X"0054",X"0055",X"0055",
X"0055",X"0056",X"0056",X"0056",X"0056",
X"0057",X"0057",X"0057",X"0058",X"0058",
X"0059",X"0059",X"005A",X"005A",X"005B",
X"005B",X"005B",X"005B",X"005C",X"005C",
X"005C",X"005D",X"005D",X"005E",X"005E",
X"005E",X"005E",X"005F",X"005F",X"0060",
X"0060",X"0061",X"0061",X"0061",X"0061",
X"0062",X"0062",X"0063",X"0063",X"0064",
X"0065",X"0064",X"0065",X"0065",X"0066",
X"0066",X"0067",X"0066",X"0067",X"0067",
X"0068",X"0069",X"0069",X"0069",X"0069",
X"006A",X"006A",X"006B",X"006B",X"006B",
X"006C",X"006C",X"006D",X"006D",X"006D",
X"006E",X"006E",X"006F",X"006F",X"006F",
X"0070",X"0070",X"0071",X"0071",X"0071",
X"0072",X"0073",X"0072",X"0073",X"0074",
X"0074",X"0075",X"0075",X"0075",X"0076",
X"0077",X"0076",X"0077",X"0078",X"0078",
X"0078",X"0079",X"0079",X"0079",X"007A",
X"007A",X"007B",X"007B",X"007B",X"007C",
X"007D",X"007D",X"007D",X"007E",X"007E",
X"007F",X"007F",X"0080",X"0080",X"0081",
X"0081",X"0081",X"0082",X"0083",X"0082",
X"0083",X"0084",X"0084",X"0084",X"0085",
X"0085",X"0086",X"0086",X"0086",X"0087",
X"0088",X"0088",X"0088",X"0089",X"0089",
X"008A",X"008B",X"008A",X"008B",X"008C",
X"008C",X"008D",X"008E",X"008D",X"008E",
X"008F",X"008F",X"0090",X"0090",X"0090",
X"0091",X"0091",X"0092",X"0092",X"0093",
X"0094",X"0093",X"0094",X"0094",X"0095",
X"0096",X"0096",X"0097",X"0097",X"0097",
X"0098",X"0098",X"0099",X"0099",X"009A",
X"009B",X"009B",X"009C",X"009B",X"009C",
X"009C",X"009D",X"009D",X"009E",X"009F",
X"009F",X"00A0",X"00A0",X"00A1",X"00A1",
X"00A1",X"00A1",X"00A2",X"00A2",X"00A3",
X"00A4",X"00A4",X"00A5",X"00A5",X"00A6",
X"00A6",X"00A7",X"00A7",X"00A8",X"00A7",
X"00A8",X"00A8",X"00A9",X"00A9",X"00AA",
X"00AA",X"00AB",X"00AB",X"00AC",X"00AC",
X"00AD",X"00AD",X"00AE",X"00AE",X"00AF",
X"00AF",X"00B0",X"00AF",X"00AF");

-------------------------------------------------------------------------------

--holds the output of the respective adders for x & y
signal xAdderOut, yAdderOut, lineAddOut : std_logic_vector( 15 downto 0 );
signal yMultByConstAddOut : std_logic_vector(19 downto 0);
signal multSinTimesYOut, multNegSinTimesXOut: std_logic_vector( 31 downto 0);

-- Intermediate signals
signal originalX, originalY, originalSinTheta, originalNegSinTheta: std_logic_vector(15 downto 0);
signal xAbsRound, yAbsRound : std_logic_vector(9 downto 0);
signal notY, notLine, extLine, negBramDataIn: std_logic_vector(15 downto 0);
signal yTenShift,yNineShift,ySevenShift,yFiveShift: std_logic_vector(19 downto 0);
signal finalX, finalY, addrAddOut, addrDrop : std_logic_vector(19 downto 0);

-- signals to control loading of registers and selection of muxes
signal
  xRegLoad,                             -- load X_REG with output of X_MUX
  yRegLoad,                             -- load Y_REG with output of Y_MUX
  addrRegLoad,                          -- load ADDR_REG with addrAddOut
  xRegInMuxSel,                         -- 0->load X from X_ADD, 1->load x_original
  yRegInMuxSel,                         -- 0->load Y from Y_ADD, 1->load y_original
  sinThetaRegLoad,                      -- load SIN_REG with original sinTheta
  negSinThetaRegLoad                    -- load NEG_SIN_REG with original negSinTheta
  : std_logic;
signal bramAddrMuxSelect: std_logic_vector(1 downto 0); -- selects which bram to load from

signal
  x,                                    -- holds the value of X_REG
  x_temp,
  y,                                    -- holds the value of Y_REG
  xAbsolute,                            -- holds the value of X_ABS_REG
  yAbsolute,                            -- holds the value of Y_ABS_REG
  sinTheta,                             -- holds the value of the SIN_REG
  negSinTheta                           -- holds the value of NEG_SIN_REG
  : std_logic_vector(15 downto 0);

-- these muxes feed the input of the x/y register inputs
-- begining of each scanline, we first load the initial x/y (from BRAM)
-- after that, we will load x/y with the output of their adders
signal
  xRegInMuxOut,                         -- output of X_REG_IN_MUX 
  yRegInMuxOut                          -- output of Y_REG_IN_MUX                          
  : std_logic_vector(15 downto 0);
signal bramAddrOffset : std_logic_vector(9 downto 0);

--state machine
type WARPER_STATES is (INIT_STATE, LOAD_X, WAIT_X, LOAD_Y, LOAD_SIN, HOLD, ADD_MULT_ONE, ADD_MULT_TWO, ADD_MULT_THREE, ADD_MULT_FOUR);
signal curState : WARPER_STATES := INIT_STATE;
signal nextState : WARPER_STATES; 



begin

  originalX <= X_RAM(conv_integer(linecount(9 downto 1)));
  originalY <= Y_RAM(conv_integer(linecount(9 downto 1)));
  originalSinTheta <= SIN_RAM(conv_integer(linecount(9 downto 1)));
  originalNegSinTheta <= not originalSinTheta + "0000000000000001";
  
  X_REG_IN_MUX: mux port map( rst, xRegInMuxSel, xAdderOut, originalX, xRegInMuxOut );
  Y_REG_IN_MUX: mux port map( rst, yRegInMuxSel, yAdderOut, originalY, yRegInMuxOut );
  BRAM_ADDR_MUX: mux4to1 port map( rst, bramAddrMuxSelect, bramAddrXOffset, bramAddrYOffset, bramAddrSinOffset, unused, bramAddrOffset);
  
  X_REG: regis port map( rst, clk, xRegLoad, xRegInMuxOut, x ); 
  Y_REG: regis port map( rst, clk, yRegLoad, yRegInMuxOut, y );
  SIN_REG: regis port map (rst, clk, sinThetaRegLoad, originalSinTheta, sinTheta);
  NEG_SIN_REG: regis port map ( rst, clk, negSinThetaRegLoad, originalNegSinTheta, negSinTheta); 
  ADDR_REG: regis20 port map( rst, clk, addrRegLoad, addrDrop, addr);

  X_MULT: mymult port map( rst, clk, sinTheta,    y, multSinTimesYOut );
  Y_MULT: mymult port map( rst, clk, negSinTheta, x, multNegSinTimesXOut );
  
  X_ADD: adder port map( rst, x, multSinTimesYOut   (31 downto 16), xAdderOut );
  Y_ADD: adder port map( rst, y, multNegSinTimesXOut(31 downto 16), yAdderOut );
  LINE_ADD: adder port map( rst, extLine, xOriginTranslateAndLineMax, lineAddOut );
  
  X_ABS_ADD: adder port map( rst, x, lineAddOut, xAbsolute );
  Y_ABS_ADD: adder port map( rst, notY, yOriginTranslatePlusOne, yAbsolute );

-- Non-16-bit adders:
  --bramAddr <= linecount(9 downto 1) + bramAddrOffset;
 -- negBramDataIn <= not bramDataIn + "0000000000000001";
  
  xAbsRound <= xAbsolute(15 downto 6) + ("000000000" & xAbsolute(5));
  yAbsRound <= yAbsolute(15 downto 6) + ("000000000" & yAbsolute(5)); --+ ("000000000" & xAbsRound(0));
  
  yMultByConstAddOut <= yNineShift + ySevenShift;  -- multiply by 640 = "0110000000"
  addrAddOut <= finalX + finalY;
  addrDrop <= addrAddOut(19 downto 1)&"0";
-- extending, inverting, chopping
  notY <= not y;
   -- (linecount+1)/2 -> fixed-pt&16-bit
  extLine <= "0" & linecount(9 downto 1) & "000000";

--  notLine <= not extLine + X"0001";
  
  --yTenShift <= yAbsRound & "0000000000";
  yNineShift  <= "0" & yAbsRound & "000000000";  -- shift by 9 for mult-by-640
  ySevenShift <= "000" & yAbsRound & "0000000";  -- again by 7
  --yFiveShift <= "00000" & yAbsRound & "00000";
  
  finalX <= "0000000000" & xAbsRound;  -- drop frac-part & extend
  finalY <= yMultByConstAddOut;  -- drop frac-part

  fsm_seq : process(rst, clk)
  begin
    if rst = '1' then
      curState <= INIT_STATE;
    elsif clk'event and clk = '1' then
      curState <= nextState;  
    end if;
  end process fsm_seq;
  
  fsm_comb : process(rst, clk, curState, pixelcount)
  begin
    xRegLoad <= '0';
    yRegLoad <= '0';
    sinThetaRegLoad <= '0';
    negSinThetaRegLoad <= '0';
    xRegInMuxSel <= '0';
    yRegInMuxSel <= '0';
    addrRegLoad <= '0';
    bramAddrMuxSelect <= "11";
    
    case curState is
      when INIT_STATE =>
        xRegInMuxSel <='1';
        yRegInMuxSel <='1';

        if (pixelcount = pixelCountSetup) then
          nextState <= LOAD_X;
        else
          nextState <= INIT_STATE;
        end if;
        
      when LOAD_X =>
        bramAddrMuxSelect <= "00";
        xRegLoad <= '1';
        xRegInMuxSel <= '1';

        nextState <= WAIT_X;

      when WAIT_X =>
        bramAddrMuxSelect <= "00";
        xRegLoad <= '1';
        xRegInMuxSel <= '1';
        
        nextState <= LOAD_Y;
        
      when LOAD_Y =>
        bramAddrMuxSelect <= "01";
        yRegLoad <= '1';
        yRegInMuxSel <= '1';
        
        nextState <= LOAD_SIN;

      when LOAD_SIN =>
        bramAddrMuxSelect <= "10";
        sinThetaRegLoad <= '1';      
        negSinThetaRegLoad <= '1';
        -- Also set to load address here, so that once x/y have propagated, the
        -- initial value of addr is valid (otherwise we would skip the first one)
        addrRegLoad <= '1';

        
        nextState <= HOLD;

      when HOLD =>
        -- Everything must be low here (it is by default)
        
        if (pixelcount = pixelCountStart) then 
          nextState <= ADD_MULT_ONE;
        else
          nextState <= HOLD;
        end if;
        
      when ADD_MULT_ONE =>
        xRegLoad <= '1';
        yRegLoad <= '1';

        nextState <= ADD_MULT_TWO;
        
      when ADD_MULT_TWO =>
        
        xRegLoad <= '0';
        yRegLoad <= '0';
        addrRegLoad <= '1';
                       
        nextState <= ADD_MULT_THREE;

        -- Because we're outputting at 640x480, but the virtual projection to
        -- 320x240, we use each pixel twice 
      when ADD_MULT_THREE =>
        addrRegLoad <= '0';
        
        nextState <= ADD_MULT_FOUR;

      when ADD_MULT_FOUR =>
        if (pixelcount >= pixelCountStop) then
          nextState <= INIT_STATE;
        else
          nextState <= ADD_MULT_ONE;
        end if;
                
      when others => null;
    end case;
  end process fsm_comb;          
  
  -- for now just set these, later they will be from output of BRAM
--  originalX           <= "0101011000" & "110000";  -- 104.746 + 240
--  originalY           <= "0001000001" & "011110";  -- 65.4665
--  originalSinTheta    <= "0000000001001101";  -- ~.001172
--  originalNegSinTheta <= "1111111110110011";  -- ~(-.001172)


  
end warper_arch;

